// streams.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

// Generates a newline character when passed to a stream.
Symbol* EOL = NULL;

// A string constant containing the character sequence
// necessary to generate a newline.
char* EOL_STRING = "\n";

// Denotes the standard input stream for the host language.
InputStream* STANDARD_INPUT = NULL;

// Denotes the standard output stream for the host language.
OutputStream* STANDARD_OUTPUT = NULL;

// Denotes the standard warning stream for the host language.
OutputStream* STANDARD_WARNING = NULL;

// Denotes the standard error stream for the host language.
OutputStream* STANDARD_ERROR = NULL;

// Holds a string containing the single quote character.
StringWrapper* SINGLE_QUOTE_STRING = NULL;

// Holds a string containing the single backquote character.
StringWrapper* SINGLE_BQUOTE_STRING = NULL;

Cons* formatMessageArguments(Cons* messageargumentstree, boolean trailingeolP) {
  { Cons* augmentedarguments = NIL;

    { Object* item = NULL;
      Cons* iter000 = messageargumentstree;

      while (!(iter000 == NIL)) {
        item = iter000->value;
        iter000 = iter000->rest;
        if (stringP(item) ||
            (item == EOL)) {
          augmentedarguments = cons(item, augmentedarguments);
        }
        else {
          augmentedarguments = cons(SINGLE_BQUOTE_STRING, augmentedarguments);
          augmentedarguments = cons(item, augmentedarguments);
          augmentedarguments = cons(SINGLE_QUOTE_STRING, augmentedarguments);
        }
      }
    }
    if (trailingeolP) {
      augmentedarguments = cons(EOL, augmentedarguments);
    }
    return (augmentedarguments->reverse());
  }
}

Object* printExpander(Cons* body) {
  // Print arguments to the standard output stream.
  return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, SYM_STREAMS_STELLA_STANDARD_OUTPUT, body->concatenate(NIL, 0)));
}

Object* informExpander(Cons* body) {
  // Print informative message, placing non-string arguments in quotes,
  // and terminating with a newline.
  body = formatMessageArguments(body, TRUE);
  return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, SYM_STREAMS_STELLA_STANDARD_OUTPUT, body->concatenate(NIL, 0)));
}

Object* warnExpander(Cons* body) {
  // Signal warning message, placing non-string arguments in quotes.
  body = formatMessageArguments(body, FALSE);
  if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_STREAMS_COMMON_LISP) {
    return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, KWD_STREAMS_WARN, body->concatenate(NIL, 0)));
  }
  else {
    return (listO(4, SYM_STREAMS_STELLA_PRINT_STREAM, SYM_STREAMS_STELLA_STANDARD_WARNING, wrapString("Warning: "), body->concatenate(cons(SYM_STREAMS_STELLA_EOL, NIL), 0)));
  }
}

Object* signalExpander(Symbol* type, Cons* body) {
  // Signal error message, placing non-string arguments in quotes.
  if (type == SYM_STREAMS_STELLA_NULL) {
    type = SYM_STREAMS_STELLA_STELLA_EXCEPTION;
  }
  if ((body->length() == 1) &&
      isaP(body->value, SGT_STREAMS_STELLA_STRING_WRAPPER)) {
    return (listO(3, SYM_STREAMS_STELLA_SIGNAL_EXCEPTION, listO(3, SYM_STREAMS_STELLA_NEW, type, listO(3, KWD_STREAMS_MESSAGE, body->value, NIL)), NIL));
  }
  else {
    { Symbol* streamvar = localGensym("STREAM");

      return (listO(5, SYM_STREAMS_STELLA_LET, cons(cons(streamvar, cons(listO(3, SYM_STREAMS_STELLA_NEW, SYM_STREAMS_STELLA_OUTPUT_STRING_STREAM, NIL), NIL)), NIL), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, streamvar, formatMessageArguments(body, FALSE)->concatenate(NIL, 0)), listO(3, SYM_STREAMS_STELLA_SIGNAL_EXCEPTION, listO(3, SYM_STREAMS_STELLA_NEW, type, listO(3, KWD_STREAMS_MESSAGE, listO(3, SYM_STREAMS_STELLA_THE_STRING, streamvar, NIL), NIL)), NIL), NIL));
    }
  }
}

Object* errorExpander(Cons* body) {
  // Signal error message, placing non-string arguments in quotes.
  return (listO(3, SYM_STREAMS_STELLA_SIGNAL, SYM_STREAMS_STELLA_STELLA_EXCEPTION, body->concatenate(NIL, 0)));
}

Object* continuableErrorExpander(Cons* body) {
  // Signal error message, placing non-string arguments in quotes.
  body = formatMessageArguments(body, FALSE);
  if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_STREAMS_JAVA) {
    if ((body->length() == 1) &&
        isaP(body->value, SGT_STREAMS_STELLA_STRING_WRAPPER)) {
      return (listO(4, SYM_STREAMS_STELLA_LET, cons(listO(3, SYM_STREAMS_STELLA_ERROR_MESSAGE_, wrapString(((StringWrapper*)(body->value))->wrapperValue), NIL), NIL), listO(4, SYM_STREAMS_STELLA_VERBATIM, KWD_STREAMS_JAVA, wrapString("Native.continuableError(error_message_)"), NIL), NIL));
    }
    else {
      { Symbol* streamvar = localGensym("STREAM");

        return (listO(5, SYM_STREAMS_STELLA_LET, cons(cons(streamvar, cons(listO(3, SYM_STREAMS_STELLA_NEW, SYM_STREAMS_STELLA_OUTPUT_STRING_STREAM, NIL), NIL)), NIL), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, streamvar, formatMessageArguments(body, FALSE)->concatenate(NIL, 0)), listO(4, SYM_STREAMS_STELLA_LET, cons(listO(3, SYM_STREAMS_STELLA_ERROR_MESSAGE_, listO(3, SYM_STREAMS_STELLA_THE_STRING, streamvar, NIL), NIL), NIL), listO(4, SYM_STREAMS_STELLA_VERBATIM, KWD_STREAMS_JAVA, wrapString("Native.continuableError(error_message_)"), NIL), NIL), NIL));
      }
    }
  }
  else {
    return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, KWD_STREAMS_CONTINUABLE_ERROR, body->concatenate(NIL, 0)));
  }
}

Object* signalReadErrorExpander(Cons* body) {
  // Specialized version of `signal' that throws a READ-EXCEPTION.
  return (listO(3, SYM_STREAMS_STELLA_SIGNAL, SYM_STREAMS_STELLA_READ_EXCEPTION, body->concatenate(NIL, 0)));
}

Cons* formatWalkMessageArguments(Cons* messageargumentstree) {
  { ConsIterator* it = messageargumentstree->allocateIterator();

    while (it->nextP()) {
      { Object* item = it->value;

        if (subtypeOfSymbolP(safePrimaryType(item))) {
          { Object* item000 = item;
            Symbol* item = ((Symbol*)(item000));

            if (!(item == EOL)) {
              it->valueSetter(listO(3, SYM_STREAMS_STELLA_DE_UGLIFY_PARSE_TREE, item, NIL));
            }
          }
        }
        else {
        }
      }
    }
  }
  return (messageargumentstree);
}

Object* walkErrorExpander(Cons* body) {
  { Cons* messagearguments = formatMessageArguments(formatWalkMessageArguments(body), FALSE);

    return (listO(5, SYM_STREAMS_STELLA_SPECIAL, cons(listO(3, SYM_STREAMS_STELLA_oPRINTREADABLYpo, SYM_STREAMS_STELLA_TRUE, NIL), NIL), cons(SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_ERROR, NIL), listO(5, SYM_STREAMS_STELLA_UNLESS, cons(SYM_STREAMS_STELLA_SUPPRESS_WARNINGSp, NIL), listO(4, SYM_STREAMS_STELLA_PRINT_ERROR_CONTEXT, wrapString(">> ERROR: "), SYM_STREAMS_STELLA_STANDARD_ERROR, NIL), listO(5, SYM_STREAMS_STELLA_PRINT_STREAM, SYM_STREAMS_STELLA_STANDARD_ERROR, SYM_STREAMS_STELLA_EOL, wrapString(" "), messagearguments->concatenate(listO(3, wrapString("."), SYM_STREAMS_STELLA_EOL, NIL), 0)), NIL), NIL));
  }
}

Object* walkInformExpander(Cons* body) {
  { Cons* messagearguments = formatMessageArguments(formatWalkMessageArguments(body), FALSE);

    return (listO(5, SYM_STREAMS_STELLA_SPECIAL, cons(listO(3, SYM_STREAMS_STELLA_oPRINTREADABLYpo, SYM_STREAMS_STELLA_TRUE, NIL), NIL), cons(SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_WARNING, NIL), listO(5, SYM_STREAMS_STELLA_UNLESS, cons(SYM_STREAMS_STELLA_SUPPRESS_WARNINGSp, NIL), listO(4, SYM_STREAMS_STELLA_PRINT_ERROR_CONTEXT, wrapString(">> WARNING: "), SYM_STREAMS_STELLA_STANDARD_OUTPUT, NIL), listO(5, SYM_STREAMS_STELLA_PRINT_STREAM, SYM_STREAMS_STELLA_STANDARD_OUTPUT, SYM_STREAMS_STELLA_EOL, wrapString(" "), messagearguments->concatenate(listO(3, wrapString("."), SYM_STREAMS_STELLA_EOL, NIL), 0)), NIL), NIL));
  }
}

Object* walkWarnExpander(Cons* body) {
  { Cons* messagearguments = formatMessageArguments(formatWalkMessageArguments(body), FALSE);

    return (listO(5, SYM_STREAMS_STELLA_SPECIAL, cons(listO(3, SYM_STREAMS_STELLA_oPRINTREADABLYpo, SYM_STREAMS_STELLA_TRUE, NIL), NIL), cons(SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_WARNING, NIL), listO(5, SYM_STREAMS_STELLA_UNLESS, cons(SYM_STREAMS_STELLA_SUPPRESS_WARNINGSp, NIL), listO(4, SYM_STREAMS_STELLA_PRINT_ERROR_CONTEXT, wrapString(">> WARNING: "), SYM_STREAMS_STELLA_STANDARD_WARNING, NIL), listO(5, SYM_STREAMS_STELLA_PRINT_STREAM, SYM_STREAMS_STELLA_STANDARD_WARNING, SYM_STREAMS_STELLA_EOL, wrapString(" "), messagearguments->concatenate(listO(3, wrapString("."), SYM_STREAMS_STELLA_EOL, NIL), 0)), NIL), NIL));
  }
}

Object* safetyExpander(IntegerWrapper* level, Object* test, Cons* body) {
  // Signal warning message, placing non-string arguments in quotes.
  if (level->wrapperValue > oSAFETYo.get()) {
    return (NULL);
  }
  body = formatMessageArguments(body, FALSE);
  return (listO(4, SYM_STREAMS_STELLA_WHEN, listO(3, SYM_STREAMS_STELLA_NOT, test, NIL), listO(4, SYM_STREAMS_STELLA_PRINT_STREAM, KWD_STREAMS_ERROR, wrapString("Safety violation: "), body->concatenate(NIL, 0)), NIL));
}

Object* evaluationErrorExpander(Cons* body) {
  { Symbol* streamvar = localGensym("STREAM");

    return (listO(8, SYM_STREAMS_STELLA_LET, cons(cons(streamvar, cons(listO(3, SYM_STREAMS_STELLA_NEW, SYM_STREAMS_STELLA_OUTPUT_STRING_STREAM, NIL), NIL)), NIL), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, streamvar, listO(3, wrapString("While evaluating '"), SYM_STREAMS_STELLA_oEVALUATIONTREEo, NIL)), listO(4, SYM_STREAMS_STELLA_WHEN, listO(3, SYM_STREAMS_STELLA_DEFINEDp, SYM_STREAMS_STELLA_oEVALUATIONPARENTTREEo, NIL), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, streamvar, listO(4, SYM_STREAMS_STELLA_EOL, wrapString("' inside '"), SYM_STREAMS_STELLA_oEVALUATIONPARENTTREEo, NIL)), NIL), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, streamvar, listO(3, wrapString("':"), SYM_STREAMS_STELLA_EOL, NIL)), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, streamvar, formatMessageArguments(body, FALSE)->concatenate(NIL, 0)), listO(3, SYM_STREAMS_STELLA_SIGNAL_EXCEPTION, listO(5, SYM_STREAMS_STELLA_NEW, SYM_STREAMS_STELLA_EVALUATION_EXCEPTION, KWD_STREAMS_MESSAGE, listO(3, SYM_STREAMS_STELLA_THE_STRING, streamvar, NIL), NIL), NIL), NIL));
  }
}

Cons* formatStringMessageArguments(Cons* messagearguments, boolean trailingeolP) {
  { Cons* formattedarguments = formatMessageArguments(messagearguments, trailingeolP);

    { ConsIterator* it = formattedarguments->allocateIterator();

      while (it->nextP()) {
        { Surrogate* testValue000 = safePrimaryType(it->value);

          if (subtypeOfStringP(testValue000)) {
          }
          else if (subtypeOfP(testValue000, SGT_STREAMS_STELLA_LITERAL_WRAPPER)) {
            it->valueSetter(wrapString(stringify(it->value)));
          }
          else {
            if (it->value == SYM_STREAMS_STELLA_EOL) {
              it->valueSetter(wrapString(EOL_STRING));
            }
            else {
              it->valueSetter(listO(3, SYM_STREAMS_STELLA_STRINGIFY, it->value, NIL));
            }
          }
        }
      }
    }
    return (formattedarguments);
  }
}

Object* messageToStringExpander(Cons* body) {
  { Cons* messagearguments = formatStringMessageArguments(body, FALSE);
    Symbol* messagevar = localGensym("MESSAGE");

    return (listO(5, SYM_STREAMS_STELLA_VRLET, cons(cons(messagevar, listO(3, SYM_STREAMS_STELLA_STRING, SYM_STREAMS_STELLA_NULL, NIL)), NIL), listO(4, SYM_STREAMS_STELLA_SPECIAL, cons(listO(3, SYM_STREAMS_STELLA_oPRINTREADABLYpo, SYM_STREAMS_STELLA_TRUE, NIL), NIL), listO(3, SYM_STREAMS_STELLA_SETQ, messagevar, cons(cons(SYM_STREAMS_STELLA_CONCATENATE, messagearguments->concatenate(NIL, 0)), NIL)), NIL), messagevar, NIL));
  }
}

Cons* expandMarkupTagFunction(Cons* tagAndStream, Cons* body, boolean xmlP) {
  { Object* tag = tagAndStream->value;
    Symbol* tagvar = NULL;
    char* tagvalue = NULL;
    Object* stream = tagAndStream->rest->value;
    Object* eolargument = tagAndStream->rest->rest->value;
    boolean eolP = ((boolean)(eolargument)) &&
        coerceWrappedBooleanToBoolean(coerceToBoolean(eolargument));

    if (body == NIL) {
      { Surrogate* testValue000 = safePrimaryType(tag);

        if (subtypeOfStringP(testValue000)) {
          { Object* tag000 = tag;
            StringWrapper* tag = ((StringWrapper*)(tag000));

            if (eolP) {
              return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, cons(wrapString(stringConcatenate("<", tag->wrapperValue, 1, (xmlP ? (char*)"/>" : (char*)">"))), cons(SYM_STREAMS_STELLA_EOL, NIL))));
            }
            else {
              return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, cons(wrapString(stringConcatenate("<", tag->wrapperValue, 1, (xmlP ? (char*)"/>" : (char*)">"))), NIL)));
            }
          }
        }
        else if (subtypeOfP(testValue000, SGT_STREAMS_STELLA_OBJECT)) {
          { Object* tag001 = tag;
            Object* tag = tag001;

            if (eolP) {
              return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(3, wrapString("<"), tag, cons((xmlP ? wrapString("/>") : wrapString(">")), cons(SYM_STREAMS_STELLA_EOL, NIL)))));
            }
            else {
              return (listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(3, wrapString("<"), tag, cons((xmlP ? wrapString("/>") : wrapString(">")), NIL))));
            }
          }
        }
        else {
          { OutputStringStream* stream000 = newOutputStringStream();

            *(stream000->nativeStream) << "`" << testValue000 << "'" << " is not a valid case option";
            throw *newStellaException(stream000->theStringReader());
          }
        }
      }
    }
    else {
      { Surrogate* testValue001 = safePrimaryType(tag);

        if (subtypeOfStringP(testValue001)) {
          { Object* tag002 = tag;
            StringWrapper* tag = ((StringWrapper*)(tag002));

            tagvalue = tag->wrapperValue;
            if (eolP) {
              return (listO(3, SYM_STREAMS_STELLA_PROGN, listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, cons(wrapString(stringConcatenate("<", tagvalue, 1, ">")), NIL)), body->concatenate(cons(listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, cons(wrapString(stringConcatenate("</", stringSubsequence(tagvalue, 0, stringPosition(tagvalue, ' ', 0)), 1, ">")), cons(SYM_STREAMS_STELLA_EOL, NIL))), NIL), 0)));
            }
            else {
              return (listO(3, SYM_STREAMS_STELLA_PROGN, listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, cons(wrapString(stringConcatenate("<", tagvalue, 1, ">")), NIL)), body->concatenate(cons(listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, cons(wrapString(stringConcatenate("</", stringSubsequence(tagvalue, 0, stringPosition(tagvalue, ' ', 0)), 1, ">")), NIL)), NIL), 0)));
            }
          }
        }
        else if (subtypeOfSymbolP(testValue001)) {
          { Object* tag003 = tag;
            Symbol* tag = ((Symbol*)(tag003));

            if (eolP) {
              return (listO(3, SYM_STREAMS_STELLA_PROGN, listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(3, wrapString("<"), tag, cons(wrapString(">"), NIL))), body->concatenate(cons(listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(5, wrapString("</"), listO(3, SYM_STREAMS_STELLA_SUBSEQUENCE, tag, listO(3, wrapInteger(0), listO(3, SYM_STREAMS_STELLA_POSITION, tag, listO(3, wrapCharacter(' '), wrapInteger(0), NIL)), NIL)), wrapString(">"), SYM_STREAMS_STELLA_EOL, NIL)), NIL), 0)));
            }
            else {
              return (listO(3, SYM_STREAMS_STELLA_PROGN, listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(3, wrapString("<"), tag, cons(wrapString(">"), NIL))), body->concatenate(cons(listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(4, wrapString("</"), listO(3, SYM_STREAMS_STELLA_SUBSEQUENCE, tag, listO(3, wrapInteger(0), listO(3, SYM_STREAMS_STELLA_POSITION, tag, listO(3, wrapCharacter(' '), wrapInteger(0), NIL)), NIL)), wrapString(">"), NIL)), NIL), 0)));
            }
          }
        }
        else if (subtypeOfP(testValue001, SGT_STREAMS_STELLA_OBJECT)) {
          { Object* tag004 = tag;
            Object* tag = tag004;

            tagvar = gensym("TAG");
            if (eolP) {
              return (listO(4, SYM_STREAMS_STELLA_LET, cons(cons(tagvar, listO(3, SYM_STREAMS_STELLA_STRING, tag, NIL)), NIL), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(3, wrapString("<"), tagvar, cons(wrapString(">"), NIL))), body->concatenate(cons(listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(5, wrapString("</"), listO(3, SYM_STREAMS_STELLA_SUBSEQUENCE, tagvar, listO(3, wrapInteger(0), listO(3, SYM_STREAMS_STELLA_POSITION, tagvar, listO(3, wrapCharacter(' '), wrapInteger(0), NIL)), NIL)), wrapString(">"), SYM_STREAMS_STELLA_EOL, NIL)), NIL), 0)));
            }
            else {
              return (listO(4, SYM_STREAMS_STELLA_LET, cons(cons(tagvar, listO(3, SYM_STREAMS_STELLA_STRING, tag, NIL)), NIL), listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(3, wrapString("<"), tagvar, cons(wrapString(">"), NIL))), body->concatenate(cons(listO(3, SYM_STREAMS_STELLA_PRINT_STREAM, stream, listO(4, wrapString("</"), listO(3, SYM_STREAMS_STELLA_SUBSEQUENCE, tagvar, listO(3, wrapInteger(0), listO(3, SYM_STREAMS_STELLA_POSITION, tagvar, listO(3, wrapCharacter(' '), wrapInteger(0), NIL)), NIL)), wrapString(">"), NIL)), NIL), 0)));
            }
          }
        }
        else {
          { OutputStringStream* stream001 = newOutputStringStream();

            *(stream001->nativeStream) << "`" << testValue001 << "'" << " is not a valid case option";
            throw *newStellaException(stream001->theStringReader());
          }
        }
      }
    }
  }
}

Object* withHtmlTagExpander(Cons* tagAndStream, Cons* body) {
  return (expandMarkupTagFunction(tagAndStream, body, FALSE));
}

Object* withXmlTagExpander(Cons* tagAndStream, Cons* body) {
  return (expandMarkupTagFunction(tagAndStream, body, TRUE));
}

boolean terminateOutputStreamP(OutputStream* self) {
  { ostream* nativeStream = self->nativeStream;

    if (!(nativeStream == NULL)) {
      delete nativeStream;
    }
    self->nativeStream = NULL;
    self->state = KWD_STREAMS_CLOSED;
    return (TRUE);
  }
}

boolean terminateInputStreamP(InputStream* self) {
  { istream* nativeStream = self->nativeStream;

    if (!(nativeStream == NULL)) {
      delete nativeStream;
    }
    self->nativeStream = NULL;
    self->state = KWD_STREAMS_CLOSED;
    return (TRUE);
  }
}

// List of file streams that are currently open.
List* oOPEN_FILE_STREAMSo = NULL;

void initializeFileOutputStream(OutputFileStream* self) {
  { char* filename = translateLogicalPathname(self->filename);
    char* ifexistsaction = self->ifExistsAction->symbolName;

    self->nativeStream = (new ofstream(filename));
    if (self->nativeStream == NULL) {
      { OutputStringStream* stream000 = newOutputStringStream();

        *(stream000->nativeStream) << "initialize-file-output-stream: Could not open " << "`" << self << "'";
        throw *newStellaException(stream000->theStringReader());
      }
    }
    self->state = KWD_STREAMS_OPEN;
    oOPEN_FILE_STREAMSo->push(self);
  }
}

boolean terminateFileOutputStreamP(OutputFileStream* self) {
  { ostream* nativeStream = self->nativeStream;

    if (!(nativeStream == NULL)) {
      delete nativeStream;
    }
    self->nativeStream = NULL;
    self->state = KWD_STREAMS_CLOSED;
    oOPEN_FILE_STREAMSo->remove(self);
    return (TRUE);
  }
}

void initializeFileInputStream(InputFileStream* self) {
  { char* filename = translateLogicalPathname(self->filename);
    char* ifnotexistsaction = self->ifNotExistsAction->symbolName;

    self->nativeStream = (new ifstream(filename));
    if (self->nativeStream == NULL) {
      { OutputStringStream* stream000 = newOutputStringStream();

        *(stream000->nativeStream) << "initialize-file-input-stream: Could not open " << "`" << self << "'";
        throw *newStellaException(stream000->theStringReader());
      }
    }
    self->state = KWD_STREAMS_OPEN;
    oOPEN_FILE_STREAMSo->push(self);
  }
}

boolean terminateFileInputStreamP(InputFileStream* self) {
  { istream* nativeStream = self->nativeStream;

    if (!(nativeStream == NULL)) {
      delete nativeStream;
    }
    self->nativeStream = NULL;
    self->state = KWD_STREAMS_CLOSED;
    oOPEN_FILE_STREAMSo->remove(self);
    return (TRUE);
  }
}

void initializeStringOutputStream(OutputStringStream* self) {
  self->nativeStream = (new (GC) ostrstream());
}

char* OutputStringStream::theStringReader() {
  { OutputStringStream* self = this;

    { ostream* nativeStream = self->nativeStream;
      char* string = NULL;

      string = ((ostrstream*)nativeStream)->str();
      if (string == NULL) {
        string = "";
      }
      return (string);
    }
  }
}

void initializeStringInputStream(InputStringStream* self) {
  { char* string = self->theString;

    self->nativeStream = (new (GC) istrstream(string));
  }
}

void closeStream(Stream* self) {
  // Close the stream `self'.
  { Surrogate* testValue000 = safePrimaryType(self);

    if (subtypeOfP(testValue000, SGT_STREAMS_STELLA_FILE_INPUT_STREAM)) {
      { Stream* self000 = self;
        InputFileStream* self = ((InputFileStream*)(self000));

        terminateFileInputStreamP(self);
      }
    }
    else if (subtypeOfP(testValue000, SGT_STREAMS_STELLA_FILE_OUTPUT_STREAM)) {
      { Stream* self001 = self;
        OutputFileStream* self = ((OutputFileStream*)(self001));

        terminateFileOutputStreamP(self);
      }
    }
    else if (subtypeOfP(testValue000, SGT_STREAMS_STELLA_INPUT_STREAM)) {
      { Stream* self002 = self;
        InputStream* self = ((InputStream*)(self002));

        terminateInputStreamP(self);
      }
    }
    else if (subtypeOfP(testValue000, SGT_STREAMS_STELLA_OUTPUT_STREAM)) {
      { Stream* self003 = self;
        OutputStream* self = ((OutputStream*)(self003));

        terminateOutputStreamP(self);
      }
    }
    else {
    }
  }
}

void flushOutput(OutputStream* self) {
  // Flush all buffered output of `self'.
  { ostream* nativeStream = self->nativeStream;

    flush(*nativeStream);
  }
}

void closeAllFiles() {
  // Close all currently open file streams.  Use for emergencies or for cleanup.
  { List* openstreams = oOPEN_FILE_STREAMSo->copy();

    { Stream* stream = NULL;
      Cons* iter000 = openstreams->theConsList;

      while (!(iter000 == NIL)) {
        stream = ((Stream*)(iter000->value));
        iter000 = iter000->rest;
        closeStream(stream);
      }
    }
    openstreams->free();
  }
}

InputStream* openNetworkStream(char* host, int port, OutputStream*& _Return1) {
  // Open a TCP/IP network stream to `host' at `port' and return the result
  // as an input/output stream pair.
  { istream* in = NULL;
    ostream* out = NULL;

    throw *newStellaException("Network streams not yet implemented for this target language");
    { InputStream* self001 = newInputStream();

      self001->nativeStream = in;
      self001->state = KWD_STREAMS_OPEN;
      { InputStream* value000 = self001;

        { OutputStream* self002 = newOutputStream();

          self002->nativeStream = out;
          self002->state = KWD_STREAMS_OPEN;
          { OutputStream* value001 = self002;

            _Return1 = value001;
            return (value000);
          }
        }
      }
    }
  }
}

char* oHTML_QUOTED_CHARACTERSo = "><&\"";

boolean htmlCharacterNeedsQuotingP(char ch) {
  return (stringMemberP(oHTML_QUOTED_CHARACTERSo, ch));
}

void writeHtmlCharacterQuotingSpecialCharacters(ostream* stream, char ch) {
  switch (ch) {
    case '>': 
      *(stream) << "&gt;";
    break;
    case '<': 
      *(stream) << "&lt;";
    break;
    case '&': 
      *(stream) << "&amp;";
    break;
    case '"': 
      *(stream) << "&quot;";
    break;
    default:
      *(stream) << ch;
    break;
  }
  return;
}

void writeHtmlQuotingSpecialCharacters(ostream* stream, char* input) {
  { char ch = NULL_CHARACTER;
    char* vector000 = input;
    int index000 = 0;
    int length000 = strlen(vector000);

    while (index000 < length000) {
      ch = vector000[index000];
      index000 = index000 + 1;
      writeHtmlCharacterQuotingSpecialCharacters(stream, ch);
    }
  }
  return;
}

void writeHtmlEscapingUrlSpecialCharacters(ostream* stream, char* input) {
  { char ch = NULL_CHARACTER;
    char* vector000 = input;
    int index000 = 0;
    int length000 = strlen(vector000);

    while (index000 < length000) {
      ch = vector000[index000];
      index000 = index000 + 1;
      switch (ch) {
        case ' ': 
          *(stream) << "%20";
        break;
        case '?': 
          *(stream) << "%3F";
        break;
        case '#': 
          *(stream) << "%23";
        break;
        case '=': 
          *(stream) << "%3D";
        break;
        case '&': 
          *(stream) << "%26";
        break;
        case '%': 
          *(stream) << "%25";
        break;
        case '+': 
          *(stream) << "%2B";
        break;
        case '<': 
          *(stream) << "%3C";
        break;
        case '>': 
          *(stream) << "%3E";
        break;
        case '/': 
          *(stream) << "%2F";
        break;
        case ':': 
          *(stream) << "%3A";
        break;
        case '"': 
          *(stream) << "%22";
        break;
        default:
          *(stream) << ch;
        break;
      }
    }
  }
  return;
}

KeyValueList* oHTML_ESCAPE_TABLEo = NULL;

char* unescapeHtmlString(char* input) {
  // Replaces HTML escape sequences such as &amp; with their associated
  // characters.
  if (!(stringPosition(input, '&', 0) != NULL_INTEGER)) {
    return (input);
  }
  { OutputStringStream* out = newOutputStringStream();
    char* escapeString = NULL;

    { char ch = NULL_CHARACTER;
      char* vector000 = input;
      int index000 = 0;
      int length000 = strlen(vector000);

      while (index000 < length000) {
        ch = vector000[index000];
        index000 = index000 + 1;
        switch (ch) {
          case '&': 
            if (escapeString != NULL) {
              throw *newStellaException("& inside escape sequence!");
            }
            escapeString = "";
          break;
          case ';': 
            if (escapeString != NULL) {
              {
                if (((boolean)(oHTML_ESCAPE_TABLEo->lookup(wrapString(escapeString))))) {
                  *(out->nativeStream) << ((CharacterWrapper*)(oHTML_ESCAPE_TABLEo->lookup(wrapString(escapeString))))->wrapperValue;
                }
                else {
                  *(out->nativeStream) << "&" << escapeString << ";";
                }
                escapeString = NULL;
              }
            }
            else {
              *(out->nativeStream) << ";";
            }
          break;
          default:
            if (escapeString != NULL) {
              escapeString = stringConcatenate(escapeString, makeString(1, ch), 0);
            }
            else {
              *(out->nativeStream) << ch;
            }
          break;
        }
      }
    }
    return (out->theStringReader());
  }
}

Object* accessStreamIteratorSlotValue(StreamIterator* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_STREAMS_STELLA_THE_STREAM) {
    if (setvalueP) {
      self->theStream = ((InputStream*)(value));
    }
    else {
      value = self->theStream;
    }
  }
  else {
    { OutputStringStream* stream000 = newOutputStringStream();

      *(stream000->nativeStream) << "`" << slotname << "'" << " is not a valid case option";
      throw *newStellaException(stream000->theStringReader());
    }
  }
  return (value);
}

SExpressionIterator* newSExpressionIterator() {
  { SExpressionIterator* self = NULL;

    self = new SExpressionIterator();
    self->firstIterationP = TRUE;
    self->value = NULL;
    self->theStream = NULL;
    return (self);
  }
}

void SExpressionIterator::free() {
  { SExpressionIterator* self = this;

    if (terminateStreamIteratorP(self)) {
      unmake(self);
    }
  }
}

Surrogate* SExpressionIterator::primaryType() {
  { SExpressionIterator* self = this;

    return (SGT_STREAMS_STELLA_S_EXPRESSION_ITERATOR);
  }
}

LineIterator* newLineIterator() {
  { LineIterator* self = NULL;

    self = new LineIterator();
    self->firstIterationP = TRUE;
    self->value = NULL;
    self->theStream = NULL;
    return (self);
  }
}

void LineIterator::free() {
  { LineIterator* self = this;

    if (terminateStreamIteratorP(self)) {
      unmake(self);
    }
  }
}

Surrogate* LineIterator::primaryType() {
  { LineIterator* self = this;

    return (SGT_STREAMS_STELLA_LINE_ITERATOR);
  }
}

CharacterIterator* newCharacterIterator() {
  { CharacterIterator* self = NULL;

    self = new CharacterIterator();
    self->firstIterationP = TRUE;
    self->value = NULL_CHARACTER;
    self->theStream = NULL;
    return (self);
  }
}

void CharacterIterator::free() {
  { CharacterIterator* self = this;

    if (terminateStreamIteratorP(self)) {
      unmake(self);
    }
  }
}

Surrogate* CharacterIterator::primaryType() {
  { CharacterIterator* self = this;

    return (SGT_STREAMS_STELLA_CHARACTER_ITERATOR);
  }
}

boolean terminateStreamIteratorP(StreamIterator* self) {
  if (((boolean)(self->theStream))) {
    self->theStream->free();
  }
  return (TRUE);
}

SExpressionIterator* sExpressions(InputStream* stream) {
  { SExpressionIterator* self000 = newSExpressionIterator();

    self000->theStream = stream;
    { SExpressionIterator* value000 = self000;

      return (value000);
    }
  }
}

boolean SExpressionIterator::nextP() {
  { SExpressionIterator* self = this;

    self->firstIterationP = FALSE;
    { Object* object = NULL;
      boolean eofP = FALSE;

      object = readSExpression(self->theStream, eofP);
      if (eofP) {
        closeStream(self->theStream);
        return (FALSE);
      }
      else {
        self->value = object;
        return (TRUE);
      }
    }
  }
}

LineIterator* lines(InputStream* stream) {
  { LineIterator* self000 = newLineIterator();

    self000->theStream = stream;
    { LineIterator* value000 = self000;

      return (value000);
    }
  }
}

boolean LineIterator::nextP() {
  { LineIterator* self = this;

    self->firstIterationP = FALSE;
    { char* line = NULL;
      boolean eofP = FALSE;

      line = readLine(self->theStream, eofP);
      if (eofP) {
        closeStream(self->theStream);
        return (FALSE);
      }
      else {
        self->value = line;
        return (TRUE);
      }
    }
  }
}

CharacterIterator* characters(InputStream* stream) {
  { CharacterIterator* self000 = newCharacterIterator();

    self000->theStream = stream;
    { CharacterIterator* value000 = self000;

      return (value000);
    }
  }
}

boolean CharacterIterator::nextP() {
  { CharacterIterator* self = this;

    self->firstIterationP = FALSE;
    { char chaR = NULL_CHARACTER;
      boolean eofP = FALSE;

      chaR = readCharacter(self->theStream, eofP);
      if (eofP) {
        closeStream(self->theStream);
        return (FALSE);
      }
      else {
        self->value = chaR;
        return (TRUE);
      }
    }
  }
}

void helpStartupStreams1() {
  {
    SYM_STREAMS_STELLA_EOL = ((Symbol*)(internRigidSymbolWrtModule("EOL", NULL, 0)));
    SYM_STREAMS_STELLA_PRINT_STREAM = ((Symbol*)(internRigidSymbolWrtModule("PRINT-STREAM", NULL, 0)));
    SYM_STREAMS_STELLA_STANDARD_OUTPUT = ((Symbol*)(internRigidSymbolWrtModule("STANDARD-OUTPUT", NULL, 0)));
    SYM_STREAMS_STELLA_PRINT = ((Symbol*)(internRigidSymbolWrtModule("PRINT", NULL, 0)));
    SYM_STREAMS_STELLA_PRINT_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("PRINT-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_INFORM = ((Symbol*)(internRigidSymbolWrtModule("INFORM", NULL, 0)));
    SYM_STREAMS_STELLA_INFORM_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("INFORM-EXPANDER", NULL, 0)));
    KWD_STREAMS_COMMON_LISP = ((Keyword*)(internRigidSymbolWrtModule("COMMON-LISP", NULL, 2)));
    KWD_STREAMS_WARN = ((Keyword*)(internRigidSymbolWrtModule("WARN", NULL, 2)));
    SYM_STREAMS_STELLA_STANDARD_WARNING = ((Symbol*)(internRigidSymbolWrtModule("STANDARD-WARNING", NULL, 0)));
    SYM_STREAMS_STELLA_WARN = ((Symbol*)(internRigidSymbolWrtModule("WARN", NULL, 0)));
    SYM_STREAMS_STELLA_WARN_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("WARN-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_NULL = ((Symbol*)(internRigidSymbolWrtModule("NULL", NULL, 0)));
    SYM_STREAMS_STELLA_STELLA_EXCEPTION = ((Symbol*)(internRigidSymbolWrtModule("STELLA-EXCEPTION", NULL, 0)));
    SGT_STREAMS_STELLA_STRING_WRAPPER = ((Surrogate*)(internRigidSymbolWrtModule("STRING-WRAPPER", NULL, 1)));
    SYM_STREAMS_STELLA_SIGNAL_EXCEPTION = ((Symbol*)(internRigidSymbolWrtModule("SIGNAL-EXCEPTION", NULL, 0)));
    SYM_STREAMS_STELLA_NEW = ((Symbol*)(internRigidSymbolWrtModule("NEW", NULL, 0)));
    KWD_STREAMS_MESSAGE = ((Keyword*)(internRigidSymbolWrtModule("MESSAGE", NULL, 2)));
    SYM_STREAMS_STELLA_LET = ((Symbol*)(internRigidSymbolWrtModule("LET", NULL, 0)));
    SYM_STREAMS_STELLA_OUTPUT_STRING_STREAM = ((Symbol*)(internRigidSymbolWrtModule("OUTPUT-STRING-STREAM", NULL, 0)));
    SYM_STREAMS_STELLA_THE_STRING = ((Symbol*)(internRigidSymbolWrtModule("THE-STRING", NULL, 0)));
    SYM_STREAMS_STELLA_SIGNAL = ((Symbol*)(internRigidSymbolWrtModule("SIGNAL", NULL, 0)));
    SYM_STREAMS_STELLA_SIGNAL_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("SIGNAL-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_ERROR = ((Symbol*)(internRigidSymbolWrtModule("ERROR", NULL, 0)));
    SYM_STREAMS_STELLA_ERROR_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("ERROR-EXPANDER", NULL, 0)));
    KWD_STREAMS_JAVA = ((Keyword*)(internRigidSymbolWrtModule("JAVA", NULL, 2)));
    SYM_STREAMS_STELLA_ERROR_MESSAGE_ = ((Symbol*)(internRigidSymbolWrtModule("ERROR_MESSAGE_", NULL, 0)));
    SYM_STREAMS_STELLA_VERBATIM = ((Symbol*)(internRigidSymbolWrtModule("VERBATIM", NULL, 0)));
    KWD_STREAMS_CONTINUABLE_ERROR = ((Keyword*)(internRigidSymbolWrtModule("CONTINUABLE-ERROR", NULL, 2)));
    SYM_STREAMS_STELLA_CONTINUABLE_ERROR = ((Symbol*)(internRigidSymbolWrtModule("CONTINUABLE-ERROR", NULL, 0)));
    SYM_STREAMS_STELLA_CONTINUABLE_ERROR_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("CONTINUABLE-ERROR-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_READ_EXCEPTION = ((Symbol*)(internRigidSymbolWrtModule("READ-EXCEPTION", NULL, 0)));
    SYM_STREAMS_STELLA_SIGNAL_READ_ERROR = ((Symbol*)(internRigidSymbolWrtModule("SIGNAL-READ-ERROR", NULL, 0)));
    SYM_STREAMS_STELLA_SIGNAL_READ_ERROR_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("SIGNAL-READ-ERROR-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_DE_UGLIFY_PARSE_TREE = ((Symbol*)(internRigidSymbolWrtModule("DE-UGLIFY-PARSE-TREE", NULL, 0)));
    SYM_STREAMS_STELLA_SPECIAL = ((Symbol*)(internRigidSymbolWrtModule("SPECIAL", NULL, 0)));
    SYM_STREAMS_STELLA_oPRINTREADABLYpo = ((Symbol*)(internRigidSymbolWrtModule("*PRINTREADABLY?*", NULL, 0)));
    SYM_STREAMS_STELLA_TRUE = ((Symbol*)(internRigidSymbolWrtModule("TRUE", NULL, 0)));
    SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_ERROR = ((Symbol*)(internRigidSymbolWrtModule("SIGNAL-TRANSLATION-ERROR", NULL, 0)));
    SYM_STREAMS_STELLA_UNLESS = ((Symbol*)(internRigidSymbolWrtModule("UNLESS", NULL, 0)));
    SYM_STREAMS_STELLA_SUPPRESS_WARNINGSp = ((Symbol*)(internRigidSymbolWrtModule("SUPPRESS-WARNINGS?", NULL, 0)));
    SYM_STREAMS_STELLA_PRINT_ERROR_CONTEXT = ((Symbol*)(internRigidSymbolWrtModule("PRINT-ERROR-CONTEXT", NULL, 0)));
    SYM_STREAMS_STELLA_STANDARD_ERROR = ((Symbol*)(internRigidSymbolWrtModule("STANDARD-ERROR", NULL, 0)));
    SYM_STREAMS_STELLA_WALK_ERROR = ((Symbol*)(internRigidSymbolWrtModule("WALK-ERROR", NULL, 0)));
    SYM_STREAMS_STELLA_WALK_ERROR_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("WALK-ERROR-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_WARNING = ((Symbol*)(internRigidSymbolWrtModule("SIGNAL-TRANSLATION-WARNING", NULL, 0)));
    SYM_STREAMS_STELLA_WALK_INFORM = ((Symbol*)(internRigidSymbolWrtModule("WALK-INFORM", NULL, 0)));
    SYM_STREAMS_STELLA_WALK_INFORM_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("WALK-INFORM-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_WALK_WARN = ((Symbol*)(internRigidSymbolWrtModule("WALK-WARN", NULL, 0)));
    SYM_STREAMS_STELLA_WALK_WARN_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("WALK-WARN-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_WHEN = ((Symbol*)(internRigidSymbolWrtModule("WHEN", NULL, 0)));
    SYM_STREAMS_STELLA_NOT = ((Symbol*)(internRigidSymbolWrtModule("NOT", NULL, 0)));
    KWD_STREAMS_ERROR = ((Keyword*)(internRigidSymbolWrtModule("ERROR", NULL, 2)));
    SYM_STREAMS_STELLA_SAFETY = ((Symbol*)(internRigidSymbolWrtModule("SAFETY", NULL, 0)));
    SYM_STREAMS_STELLA_SAFETY_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("SAFETY-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_oEVALUATIONTREEo = ((Symbol*)(internRigidSymbolWrtModule("*EVALUATIONTREE*", NULL, 0)));
    SYM_STREAMS_STELLA_DEFINEDp = ((Symbol*)(internRigidSymbolWrtModule("DEFINED?", NULL, 0)));
    SYM_STREAMS_STELLA_oEVALUATIONPARENTTREEo = ((Symbol*)(internRigidSymbolWrtModule("*EVALUATIONPARENTTREE*", NULL, 0)));
    SYM_STREAMS_STELLA_EVALUATION_EXCEPTION = ((Symbol*)(internRigidSymbolWrtModule("EVALUATION-EXCEPTION", NULL, 0)));
    SYM_STREAMS_STELLA_EVALUATION_ERROR = ((Symbol*)(internRigidSymbolWrtModule("EVALUATION-ERROR", NULL, 0)));
  }
}

void helpStartupStreams2() {
  {
    SYM_STREAMS_STELLA_EVALUATION_ERROR_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("EVALUATION-ERROR-EXPANDER", NULL, 0)));
    SGT_STREAMS_STELLA_LITERAL_WRAPPER = ((Surrogate*)(internRigidSymbolWrtModule("LITERAL-WRAPPER", NULL, 1)));
    SYM_STREAMS_STELLA_STRINGIFY = ((Symbol*)(internRigidSymbolWrtModule("STRINGIFY", NULL, 0)));
    SYM_STREAMS_STELLA_VRLET = ((Symbol*)(internRigidSymbolWrtModule("VRLET", NULL, 0)));
    SYM_STREAMS_STELLA_STRING = ((Symbol*)(internRigidSymbolWrtModule("STRING", NULL, 0)));
    SYM_STREAMS_STELLA_SETQ = ((Symbol*)(internRigidSymbolWrtModule("SETQ", NULL, 0)));
    SYM_STREAMS_STELLA_CONCATENATE = ((Symbol*)(internRigidSymbolWrtModule("CONCATENATE", NULL, 0)));
    SYM_STREAMS_STELLA_MESSAGE_TO_STRING = ((Symbol*)(internRigidSymbolWrtModule("MESSAGE-TO-STRING", NULL, 0)));
    SYM_STREAMS_STELLA_MESSAGE_TO_STRING_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("MESSAGE-TO-STRING-EXPANDER", NULL, 0)));
    SGT_STREAMS_STELLA_OBJECT = ((Surrogate*)(internRigidSymbolWrtModule("OBJECT", NULL, 1)));
    SYM_STREAMS_STELLA_PROGN = ((Symbol*)(internRigidSymbolWrtModule("PROGN", NULL, 0)));
    SYM_STREAMS_STELLA_SUBSEQUENCE = ((Symbol*)(internRigidSymbolWrtModule("SUBSEQUENCE", NULL, 0)));
    SYM_STREAMS_STELLA_POSITION = ((Symbol*)(internRigidSymbolWrtModule("POSITION", NULL, 0)));
    SYM_STREAMS_STELLA_WITH_HTML_TAG = ((Symbol*)(internRigidSymbolWrtModule("WITH-HTML-TAG", NULL, 0)));
    SYM_STREAMS_STELLA_WITH_HTML_TAG_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("WITH-HTML-TAG-EXPANDER", NULL, 0)));
    SYM_STREAMS_STELLA_WITH_XML_TAG = ((Symbol*)(internRigidSymbolWrtModule("WITH-XML-TAG", NULL, 0)));
    SYM_STREAMS_STELLA_WITH_XML_TAG_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("WITH-XML-TAG-EXPANDER", NULL, 0)));
    KWD_STREAMS_CLOSED = ((Keyword*)(internRigidSymbolWrtModule("CLOSED", NULL, 2)));
    KWD_STREAMS_OPEN = ((Keyword*)(internRigidSymbolWrtModule("OPEN", NULL, 2)));
    SGT_STREAMS_STELLA_FILE_INPUT_STREAM = ((Surrogate*)(internRigidSymbolWrtModule("FILE-INPUT-STREAM", NULL, 1)));
    SGT_STREAMS_STELLA_FILE_OUTPUT_STREAM = ((Surrogate*)(internRigidSymbolWrtModule("FILE-OUTPUT-STREAM", NULL, 1)));
    SGT_STREAMS_STELLA_INPUT_STREAM = ((Surrogate*)(internRigidSymbolWrtModule("INPUT-STREAM", NULL, 1)));
    SGT_STREAMS_STELLA_OUTPUT_STREAM = ((Surrogate*)(internRigidSymbolWrtModule("OUTPUT-STREAM", NULL, 1)));
    SYM_STREAMS_STELLA_THE_STREAM = ((Symbol*)(internRigidSymbolWrtModule("THE-STREAM", NULL, 0)));
    SGT_STREAMS_STELLA_S_EXPRESSION_ITERATOR = ((Surrogate*)(internRigidSymbolWrtModule("S-EXPRESSION-ITERATOR", NULL, 1)));
    SGT_STREAMS_STELLA_LINE_ITERATOR = ((Surrogate*)(internRigidSymbolWrtModule("LINE-ITERATOR", NULL, 1)));
    SGT_STREAMS_STELLA_CHARACTER_ITERATOR = ((Surrogate*)(internRigidSymbolWrtModule("CHARACTER-ITERATOR", NULL, 1)));
    SYM_STREAMS_STELLA_STARTUP_STREAMS = ((Symbol*)(internRigidSymbolWrtModule("STARTUP-STREAMS", NULL, 0)));
    SYM_STREAMS_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(internRigidSymbolWrtModule("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void helpStartupStreams3() {
  {
    defineFunctionObject("FORMAT-MESSAGE-ARGUMENTS", "(DEFUN (FORMAT-MESSAGE-ARGUMENTS CONS) ((MESSAGEARGUMENTSTREE CONS) (TRAILINGEOL? BOOLEAN)))", ((cpp_function_code)(&formatMessageArguments)), NULL);
    defineFunctionObject("PRINT-EXPANDER", "(DEFUN (PRINT-EXPANDER OBJECT) (|&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Print arguments to the standard output stream.\")", ((cpp_function_code)(&printExpander)), NULL);
    defineFunctionObject("INFORM-EXPANDER", "(DEFUN (INFORM-EXPANDER OBJECT) (|&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Print informative message, placing non-string arguments in quotes,\nand terminating with a newline.\")", ((cpp_function_code)(&informExpander)), NULL);
    defineFunctionObject("WARN-EXPANDER", "(DEFUN (WARN-EXPANDER OBJECT) (|&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Signal warning message, placing non-string arguments in quotes.\")", ((cpp_function_code)(&warnExpander)), NULL);
    defineFunctionObject("SIGNAL-EXPANDER", "(DEFUN (SIGNAL-EXPANDER OBJECT) ((TYPE SYMBOL) |&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Signal error message, placing non-string arguments in quotes.\")", ((cpp_function_code)(&signalExpander)), NULL);
    defineFunctionObject("ERROR-EXPANDER", "(DEFUN (ERROR-EXPANDER OBJECT) (|&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Signal error message, placing non-string arguments in quotes.\")", ((cpp_function_code)(&errorExpander)), NULL);
    defineFunctionObject("CONTINUABLE-ERROR-EXPANDER", "(DEFUN (CONTINUABLE-ERROR-EXPANDER OBJECT) (|&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Signal error message, placing non-string arguments in quotes.\")", ((cpp_function_code)(&continuableErrorExpander)), NULL);
    defineFunctionObject("SIGNAL-READ-ERROR-EXPANDER", "(DEFUN (SIGNAL-READ-ERROR-EXPANDER OBJECT) (|&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Specialized version of `signal' that throws a READ-EXCEPTION.\")", ((cpp_function_code)(&signalReadErrorExpander)), NULL);
    defineFunctionObject("FORMAT-WALK-MESSAGE-ARGUMENTS", "(DEFUN (FORMAT-WALK-MESSAGE-ARGUMENTS CONS) ((MESSAGEARGUMENTSTREE CONS)))", ((cpp_function_code)(&formatWalkMessageArguments)), NULL);
    defineFunctionObject("WALK-ERROR-EXPANDER", "(DEFUN (WALK-ERROR-EXPANDER OBJECT) (|&BODY| (BODY CONS)))", ((cpp_function_code)(&walkErrorExpander)), NULL);
    defineFunctionObject("WALK-INFORM-EXPANDER", "(DEFUN (WALK-INFORM-EXPANDER OBJECT) (|&BODY| (BODY CONS)))", ((cpp_function_code)(&walkInformExpander)), NULL);
    defineFunctionObject("WALK-WARN-EXPANDER", "(DEFUN (WALK-WARN-EXPANDER OBJECT) (|&BODY| (BODY CONS)))", ((cpp_function_code)(&walkWarnExpander)), NULL);
    defineFunctionObject("SAFETY-EXPANDER", "(DEFUN (SAFETY-EXPANDER OBJECT) ((LEVEL INTEGER-WRAPPER) (TEST OBJECT) |&BODY| (BODY CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Signal warning message, placing non-string arguments in quotes.\")", ((cpp_function_code)(&safetyExpander)), NULL);
    defineFunctionObject("EVALUATION-ERROR-EXPANDER", "(DEFUN (EVALUATION-ERROR-EXPANDER OBJECT) (|&BODY| (BODY CONS)))", ((cpp_function_code)(&evaluationErrorExpander)), NULL);
    defineFunctionObject("FORMAT-STRING-MESSAGE-ARGUMENTS", "(DEFUN (FORMAT-STRING-MESSAGE-ARGUMENTS CONS) ((MESSAGEARGUMENTS CONS) (TRAILINGEOL? BOOLEAN)))", ((cpp_function_code)(&formatStringMessageArguments)), NULL);
    defineFunctionObject("MESSAGE-TO-STRING-EXPANDER", "(DEFUN (MESSAGE-TO-STRING-EXPANDER OBJECT) (|&BODY| (BODY CONS)))", ((cpp_function_code)(&messageToStringExpander)), NULL);
    defineFunctionObject("EXPAND-MARKUP-TAG-FUNCTION", "(DEFUN (EXPAND-MARKUP-TAG-FUNCTION CONS) ((TAG-AND-STREAM CONS) (BODY CONS) (XML? BOOLEAN)))", ((cpp_function_code)(&expandMarkupTagFunction)), NULL);
    defineFunctionObject("WITH-HTML-TAG-EXPANDER", "(DEFUN (WITH-HTML-TAG-EXPANDER OBJECT) ((TAG-AND-STREAM CONS) |&BODY| (BODY CONS)))", ((cpp_function_code)(&withHtmlTagExpander)), NULL);
    defineFunctionObject("WITH-XML-TAG-EXPANDER", "(DEFUN (WITH-XML-TAG-EXPANDER OBJECT) ((TAG-AND-STREAM CONS) |&BODY| (BODY CONS)))", ((cpp_function_code)(&withXmlTagExpander)), NULL);
    defineFunctionObject("TERMINATE-OUTPUT-STREAM?", "(DEFUN (TERMINATE-OUTPUT-STREAM? BOOLEAN) ((SELF OUTPUT-STREAM)))", ((cpp_function_code)(&terminateOutputStreamP)), NULL);
    defineFunctionObject("TERMINATE-INPUT-STREAM?", "(DEFUN (TERMINATE-INPUT-STREAM? BOOLEAN) ((SELF INPUT-STREAM)))", ((cpp_function_code)(&terminateInputStreamP)), NULL);
    defineFunctionObject("INITIALIZE-FILE-OUTPUT-STREAM", "(DEFUN INITIALIZE-FILE-OUTPUT-STREAM ((SELF FILE-OUTPUT-STREAM)))", ((cpp_function_code)(&initializeFileOutputStream)), NULL);
    defineFunctionObject("TERMINATE-FILE-OUTPUT-STREAM?", "(DEFUN (TERMINATE-FILE-OUTPUT-STREAM? BOOLEAN) ((SELF FILE-OUTPUT-STREAM)))", ((cpp_function_code)(&terminateFileOutputStreamP)), NULL);
    defineFunctionObject("INITIALIZE-FILE-INPUT-STREAM", "(DEFUN INITIALIZE-FILE-INPUT-STREAM ((SELF FILE-INPUT-STREAM)))", ((cpp_function_code)(&initializeFileInputStream)), NULL);
    defineFunctionObject("TERMINATE-FILE-INPUT-STREAM?", "(DEFUN (TERMINATE-FILE-INPUT-STREAM? BOOLEAN) ((SELF FILE-INPUT-STREAM)))", ((cpp_function_code)(&terminateFileInputStreamP)), NULL);
    defineFunctionObject("INITIALIZE-STRING-OUTPUT-STREAM", "(DEFUN INITIALIZE-STRING-OUTPUT-STREAM ((SELF STRING-OUTPUT-STREAM)))", ((cpp_function_code)(&initializeStringOutputStream)), NULL);
    defineMethodObject("(DEFMETHOD (THE-STRING-READER STRING) ((SELF STRING-OUTPUT-STREAM)))", ((cpp_method_code)(&OutputStringStream::theStringReader)), ((cpp_method_code)(NULL)));
    defineFunctionObject("INITIALIZE-STRING-INPUT-STREAM", "(DEFUN INITIALIZE-STRING-INPUT-STREAM ((SELF STRING-INPUT-STREAM)))", ((cpp_function_code)(&initializeStringInputStream)), NULL);
    defineFunctionObject("CLOSE-STREAM", "(DEFUN CLOSE-STREAM ((SELF STREAM)) :DOCUMENTATION \"Close the stream `self'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&closeStream)), NULL);
    defineFunctionObject("FLUSH-OUTPUT", "(DEFUN FLUSH-OUTPUT ((SELF OUTPUT-STREAM)) :DOCUMENTATION \"Flush all buffered output of `self'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&flushOutput)), NULL);
    defineFunctionObject("CLOSE-ALL-FILES", "(DEFUN CLOSE-ALL-FILES () :DOCUMENTATION \"Close all currently open file streams.  Use for emergencies or for cleanup.\" :PUBLIC? TRUE)", ((cpp_function_code)(&closeAllFiles)), NULL);
    defineFunctionObject("OPEN-NETWORK-STREAM", "(DEFUN (OPEN-NETWORK-STREAM INPUT-STREAM OUTPUT-STREAM) ((HOST STRING) (PORT INTEGER)) :DOCUMENTATION \"Open a TCP/IP network stream to `host' at `port' and return the result\nas an input/output stream pair.\" :PUBLIC? TRUE)", ((cpp_function_code)(&openNetworkStream)), NULL);
    defineFunctionObject("HTML-CHARACTER-NEEDS-QUOTING?", "(DEFUN (HTML-CHARACTER-NEEDS-QUOTING? BOOLEAN) ((CH CHARACTER)))", ((cpp_function_code)(&htmlCharacterNeedsQuotingP)), NULL);
    defineFunctionObject("WRITE-HTML-CHARACTER-QUOTING-SPECIAL-CHARACTERS", "(DEFUN WRITE-HTML-CHARACTER-QUOTING-SPECIAL-CHARACTERS ((STREAM NATIVE-OUTPUT-STREAM) (CH CHARACTER)) :GLOBALLY-INLINE? TRUE (CASE CH (#\\> (PRINT-STREAM STREAM \"&gt;\")) (#\\< (PRINT-STREAM STREAM \"&lt;\")) (#\\& (PRINT-STREAM STREAM \"&amp;\")) (#\\\" (PRINT-STREAM STREAM \"&quot;\")) (OTHERWISE (PRINT-STREAM STREAM CH))) (RETURN))", ((cpp_function_code)(&writeHtmlCharacterQuotingSpecialCharacters)), NULL);
    defineFunctionObject("WRITE-HTML-QUOTING-SPECIAL-CHARACTERS", "(DEFUN WRITE-HTML-QUOTING-SPECIAL-CHARACTERS ((STREAM NATIVE-OUTPUT-STREAM) (INPUT STRING)))", ((cpp_function_code)(&writeHtmlQuotingSpecialCharacters)), NULL);
    defineFunctionObject("WRITE-HTML-ESCAPING-URL-SPECIAL-CHARACTERS", "(DEFUN WRITE-HTML-ESCAPING-URL-SPECIAL-CHARACTERS ((STREAM NATIVE-OUTPUT-STREAM) (INPUT STRING)))", ((cpp_function_code)(&writeHtmlEscapingUrlSpecialCharacters)), NULL);
    defineFunctionObject("UNESCAPE-HTML-STRING", "(DEFUN (UNESCAPE-HTML-STRING STRING) ((INPUT STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Replaces HTML escape sequences such as &amp; with their associated\ncharacters.\")", ((cpp_function_code)(&unescapeHtmlString)), NULL);
    defineFunctionObject("TERMINATE-STREAM-ITERATOR?", "(DEFUN (TERMINATE-STREAM-ITERATOR? BOOLEAN) ((SELF STREAM-ITERATOR)))", ((cpp_function_code)(&terminateStreamIteratorP)), NULL);
    defineFunctionObject("S-EXPRESSIONS", "(DEFUN (S-EXPRESSIONS S-EXPRESSION-ITERATOR) ((STREAM INPUT-STREAM)) :PUBLIC? TRUE)", ((cpp_function_code)(&sExpressions)), NULL);
    defineMethodObject("(DEFMETHOD (NEXT? BOOLEAN) ((SELF S-EXPRESSION-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&SExpressionIterator::nextP)), ((cpp_method_code)(NULL)));
    defineFunctionObject("LINES", "(DEFUN (LINES LINE-ITERATOR) ((STREAM INPUT-STREAM)) :PUBLIC? TRUE)", ((cpp_function_code)(&lines)), NULL);
    defineMethodObject("(DEFMETHOD (NEXT? BOOLEAN) ((SELF LINE-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&LineIterator::nextP)), ((cpp_method_code)(NULL)));
    defineFunctionObject("CHARACTERS", "(DEFUN (CHARACTERS CHARACTER-ITERATOR) ((STREAM INPUT-STREAM)) :PUBLIC? TRUE)", ((cpp_function_code)(&characters)), NULL);
    defineMethodObject("(DEFMETHOD (NEXT? BOOLEAN) ((SELF CHARACTER-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&CharacterIterator::nextP)), ((cpp_method_code)(NULL)));
    defineFunctionObject("STARTUP-STREAMS", "(DEFUN STARTUP-STREAMS () :PUBLIC? TRUE)", ((cpp_function_code)(&startupStreams)), NULL);
    { MethodSlot* function = lookupFunction(SYM_STREAMS_STELLA_STARTUP_STREAMS);

      setDynamicSlotValue(function->dynamicSlots, SYM_STREAMS_STELLA_METHOD_STARTUP_CLASSNAME, wrapString("Startup-Streams"), NULL_STRING_WRAPPER);
    }
  }
}

void startupStreams() {
  if (currentStartupTimePhaseP(0)) {
    { OutputStream* self001 = newOutputStream();

      self001->nativeStream = &cout;
      STANDARD_OUTPUT = self001;
    }
    if (!(((boolean)(STANDARD_WARNING)))) {
      { OutputStream* self002 = newOutputStream();

        self002->nativeStream = &cerr;
        STANDARD_WARNING = self002;
      }
    }
    { OutputStream* self003 = newOutputStream();

      self003->nativeStream = &cerr;
      STANDARD_ERROR = self003;
    }
  }
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (currentStartupTimePhaseP(2)) {
      helpStartupStreams1();
      helpStartupStreams2();
    }
    if (currentStartupTimePhaseP(4)) {
      EOL = SYM_STREAMS_STELLA_EOL;
      { InputStream* self000 = newInputStream();

        self000->nativeStream = &cin;
        STANDARD_INPUT = self000;
      }
      SINGLE_QUOTE_STRING = wrapString("'");
      SINGLE_BQUOTE_STRING = wrapString("`");
      oOPEN_FILE_STREAMSo = newList();
      oHTML_ESCAPE_TABLEo = newKeyValueList();
    }
    if (currentStartupTimePhaseP(5)) {
      { Class* clasS = defineClassFromStringifiedSource("STREAM-ITERATOR", "(DEFCLASS STREAM-ITERATOR (ABSTRACT-ITERATOR) :PUBLIC? TRUE :ABSTRACT? TRUE :DOCUMENTATION \"Iterator that yields tokens from an input stream.\" :SLOTS ((THE-STREAM :TYPE INPUT-STREAM)) :TERMINATOR TERMINATE-STREAM-ITERATOR?)");

        clasS->classSlotAccessorCode = ((cpp_function_code)(&accessStreamIteratorSlotValue));
      }
      { Class* clasS = defineClassFromStringifiedSource("S-EXPRESSION-ITERATOR", "(DEFCLASS S-EXPRESSION-ITERATOR (STREAM-ITERATOR) :PUBLIC? TRUE :DOCUMENTATION \"Iterator that yields s-expressions from an input stream.\" :PARAMETERS ((ANY-VALUE :TYPE OBJECT)))");

        clasS->classConstructorCode = ((cpp_function_code)(&newSExpressionIterator));
      }
      { Class* clasS = defineClassFromStringifiedSource("LINE-ITERATOR", "(DEFCLASS LINE-ITERATOR (STREAM-ITERATOR) :PUBLIC? TRUE :DOCUMENTATION \"Iterator that yields lines from an input stream.\" :PARAMETERS ((ANY-VALUE :TYPE STRING)))");

        clasS->classConstructorCode = ((cpp_function_code)(&newLineIterator));
      }
      { Class* clasS = defineClassFromStringifiedSource("CHARACTER-ITERATOR", "(DEFCLASS CHARACTER-ITERATOR (STREAM-ITERATOR) :PUBLIC? TRUE :DOCUMENTATION \"Iterator that yields characters from an input stream.\" :PARAMETERS ((ANY-VALUE :TYPE CHARACTER)))");

        clasS->classConstructorCode = ((cpp_function_code)(&newCharacterIterator));
      }
    }
    if (currentStartupTimePhaseP(6)) {
      finalizeClasses();
    }
    if (currentStartupTimePhaseP(7)) {
      helpStartupStreams3();
    }
    if (currentStartupTimePhaseP(8)) {
      finalizeSlots();
      cleanupUnfinalizedClasses();
    }
    if (currentStartupTimePhaseP(9)) {
      defineStellaGlobalVariableFromStringifiedSource("(DEFGLOBAL EOL SYMBOL (QUOTE EOL) :DOCUMENTATION \"Generates a newline character when passed to a stream.\" :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFGLOBAL EOL-STRING STRING \"\n\" :DOCUMENTATION \"A string constant containing the character sequence\nnecessary to generate a newline.\" :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFGLOBAL STANDARD-INPUT INPUT-STREAM NULL :DOCUMENTATION \"Denotes the standard input stream for the host language.\" :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFGLOBAL STANDARD-OUTPUT OUTPUT-STREAM NULL :DOCUMENTATION \"Denotes the standard output stream for the host language.\" :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFGLOBAL STANDARD-WARNING OUTPUT-STREAM NULL :DOCUMENTATION \"Denotes the standard warning stream for the host language.\" :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFGLOBAL STANDARD-ERROR OUTPUT-STREAM NULL :DOCUMENTATION \"Denotes the standard error stream for the host language.\" :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFGLOBAL SINGLE-QUOTE-STRING STRING-WRAPPER (WRAP-LITERAL \"'\") :DOCUMENTATION \"Holds a string containing the single quote character.\")");
      defineStellaGlobalVariableFromStringifiedSource("(DEFGLOBAL SINGLE-BQUOTE-STRING STRING-WRAPPER (WRAP-LITERAL \"`\") :DOCUMENTATION \"Holds a string containing the single backquote character.\")");
      storeMacro(SYM_STREAMS_STELLA_PRINT, SYM_STREAMS_STELLA_PRINT_EXPANDER, ((cpp_function_code)(&printExpander)));
      storeMacro(SYM_STREAMS_STELLA_INFORM, SYM_STREAMS_STELLA_INFORM_EXPANDER, ((cpp_function_code)(&informExpander)));
      storeMacro(SYM_STREAMS_STELLA_WARN, SYM_STREAMS_STELLA_WARN_EXPANDER, ((cpp_function_code)(&warnExpander)));
      storeMacro(SYM_STREAMS_STELLA_SIGNAL, SYM_STREAMS_STELLA_SIGNAL_EXPANDER, ((cpp_function_code)(&signalExpander)));
      storeMacro(SYM_STREAMS_STELLA_ERROR, SYM_STREAMS_STELLA_ERROR_EXPANDER, ((cpp_function_code)(&errorExpander)));
      storeMacro(SYM_STREAMS_STELLA_CONTINUABLE_ERROR, SYM_STREAMS_STELLA_CONTINUABLE_ERROR_EXPANDER, ((cpp_function_code)(&continuableErrorExpander)));
      storeMacro(SYM_STREAMS_STELLA_SIGNAL_READ_ERROR, SYM_STREAMS_STELLA_SIGNAL_READ_ERROR_EXPANDER, ((cpp_function_code)(&signalReadErrorExpander)));
      storeMacro(SYM_STREAMS_STELLA_WALK_ERROR, SYM_STREAMS_STELLA_WALK_ERROR_EXPANDER, ((cpp_function_code)(&walkErrorExpander)));
      storeMacro(SYM_STREAMS_STELLA_WALK_INFORM, SYM_STREAMS_STELLA_WALK_INFORM_EXPANDER, ((cpp_function_code)(&walkInformExpander)));
      storeMacro(SYM_STREAMS_STELLA_WALK_WARN, SYM_STREAMS_STELLA_WALK_WARN_EXPANDER, ((cpp_function_code)(&walkWarnExpander)));
      storeMacro(SYM_STREAMS_STELLA_SAFETY, SYM_STREAMS_STELLA_SAFETY_EXPANDER, ((cpp_function_code)(&safetyExpander)));
      storeMacro(SYM_STREAMS_STELLA_EVALUATION_ERROR, SYM_STREAMS_STELLA_EVALUATION_ERROR_EXPANDER, ((cpp_function_code)(&evaluationErrorExpander)));
      storeMacro(SYM_STREAMS_STELLA_MESSAGE_TO_STRING, SYM_STREAMS_STELLA_MESSAGE_TO_STRING_EXPANDER, ((cpp_function_code)(&messageToStringExpander)));
      storeMacro(SYM_STREAMS_STELLA_WITH_HTML_TAG, SYM_STREAMS_STELLA_WITH_HTML_TAG_EXPANDER, ((cpp_function_code)(&withHtmlTagExpander)));
      storeMacro(SYM_STREAMS_STELLA_WITH_XML_TAG, SYM_STREAMS_STELLA_WITH_XML_TAG_EXPANDER, ((cpp_function_code)(&withXmlTagExpander)));
      defineStellaGlobalVariableFromStringifiedSource("(DEFGLOBAL *OPEN-FILE-STREAMS* (LIST OF STREAM) (NEW (LIST OF STREAM)) :DOCUMENTATION \"List of file streams that are currently open.\")");
      defineStellaGlobalVariableFromStringifiedSource("(DEFGLOBAL *HTML-QUOTED-CHARACTERS* STRING \"><&\\\"\")");
      defineStellaGlobalVariableFromStringifiedSource("(DEFGLOBAL *HTML-ESCAPE-TABLE* KEY-VALUE-LIST (NEW KEY-VALUE-LIST))");
      oHTML_ESCAPE_TABLEo->insertAt(wrapString("lt"), wrapCharacter('<'));
      oHTML_ESCAPE_TABLEo->insertAt(wrapString("gt"), wrapCharacter('>'));
      oHTML_ESCAPE_TABLEo->insertAt(wrapString("amp"), wrapCharacter('&'));
      oHTML_ESCAPE_TABLEo->insertAt(wrapString("quot"), wrapCharacter('"'));
      oHTML_ESCAPE_TABLEo->insertAt(wrapString("apos"), wrapCharacter('\''));
      oHTML_ESCAPE_TABLEo->insertAt(wrapString("nbsp"), wrapCharacter(' '));
      oHTML_ESCAPE_TABLEo->insertAt(wrapString("NBSP"), wrapCharacter(' '));
    }
  }
}

Symbol* SYM_STREAMS_STELLA_EOL = NULL;

Symbol* SYM_STREAMS_STELLA_PRINT_STREAM = NULL;

Symbol* SYM_STREAMS_STELLA_STANDARD_OUTPUT = NULL;

Symbol* SYM_STREAMS_STELLA_PRINT = NULL;

Symbol* SYM_STREAMS_STELLA_PRINT_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_INFORM = NULL;

Symbol* SYM_STREAMS_STELLA_INFORM_EXPANDER = NULL;

Keyword* KWD_STREAMS_COMMON_LISP = NULL;

Keyword* KWD_STREAMS_WARN = NULL;

Symbol* SYM_STREAMS_STELLA_STANDARD_WARNING = NULL;

Symbol* SYM_STREAMS_STELLA_WARN = NULL;

Symbol* SYM_STREAMS_STELLA_WARN_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_NULL = NULL;

Symbol* SYM_STREAMS_STELLA_STELLA_EXCEPTION = NULL;

Surrogate* SGT_STREAMS_STELLA_STRING_WRAPPER = NULL;

Symbol* SYM_STREAMS_STELLA_SIGNAL_EXCEPTION = NULL;

Symbol* SYM_STREAMS_STELLA_NEW = NULL;

Keyword* KWD_STREAMS_MESSAGE = NULL;

Symbol* SYM_STREAMS_STELLA_LET = NULL;

Symbol* SYM_STREAMS_STELLA_OUTPUT_STRING_STREAM = NULL;

Symbol* SYM_STREAMS_STELLA_THE_STRING = NULL;

Symbol* SYM_STREAMS_STELLA_SIGNAL = NULL;

Symbol* SYM_STREAMS_STELLA_SIGNAL_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_ERROR_EXPANDER = NULL;

Keyword* KWD_STREAMS_JAVA = NULL;

Symbol* SYM_STREAMS_STELLA_ERROR_MESSAGE_ = NULL;

Symbol* SYM_STREAMS_STELLA_VERBATIM = NULL;

Keyword* KWD_STREAMS_CONTINUABLE_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_CONTINUABLE_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_CONTINUABLE_ERROR_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_READ_EXCEPTION = NULL;

Symbol* SYM_STREAMS_STELLA_SIGNAL_READ_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_SIGNAL_READ_ERROR_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_DE_UGLIFY_PARSE_TREE = NULL;

Symbol* SYM_STREAMS_STELLA_SPECIAL = NULL;

Symbol* SYM_STREAMS_STELLA_oPRINTREADABLYpo = NULL;

Symbol* SYM_STREAMS_STELLA_TRUE = NULL;

Symbol* SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_UNLESS = NULL;

Symbol* SYM_STREAMS_STELLA_SUPPRESS_WARNINGSp = NULL;

Symbol* SYM_STREAMS_STELLA_PRINT_ERROR_CONTEXT = NULL;

Symbol* SYM_STREAMS_STELLA_STANDARD_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_WALK_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_WALK_ERROR_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_WARNING = NULL;

Symbol* SYM_STREAMS_STELLA_WALK_INFORM = NULL;

Symbol* SYM_STREAMS_STELLA_WALK_INFORM_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_WALK_WARN = NULL;

Symbol* SYM_STREAMS_STELLA_WALK_WARN_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_WHEN = NULL;

Symbol* SYM_STREAMS_STELLA_NOT = NULL;

Keyword* KWD_STREAMS_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_SAFETY = NULL;

Symbol* SYM_STREAMS_STELLA_SAFETY_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_oEVALUATIONTREEo = NULL;

Symbol* SYM_STREAMS_STELLA_DEFINEDp = NULL;

Symbol* SYM_STREAMS_STELLA_oEVALUATIONPARENTTREEo = NULL;

Symbol* SYM_STREAMS_STELLA_EVALUATION_EXCEPTION = NULL;

Symbol* SYM_STREAMS_STELLA_EVALUATION_ERROR = NULL;

Symbol* SYM_STREAMS_STELLA_EVALUATION_ERROR_EXPANDER = NULL;

Surrogate* SGT_STREAMS_STELLA_LITERAL_WRAPPER = NULL;

Symbol* SYM_STREAMS_STELLA_STRINGIFY = NULL;

Symbol* SYM_STREAMS_STELLA_VRLET = NULL;

Symbol* SYM_STREAMS_STELLA_STRING = NULL;

Symbol* SYM_STREAMS_STELLA_SETQ = NULL;

Symbol* SYM_STREAMS_STELLA_CONCATENATE = NULL;

Symbol* SYM_STREAMS_STELLA_MESSAGE_TO_STRING = NULL;

Symbol* SYM_STREAMS_STELLA_MESSAGE_TO_STRING_EXPANDER = NULL;

Surrogate* SGT_STREAMS_STELLA_OBJECT = NULL;

Symbol* SYM_STREAMS_STELLA_PROGN = NULL;

Symbol* SYM_STREAMS_STELLA_SUBSEQUENCE = NULL;

Symbol* SYM_STREAMS_STELLA_POSITION = NULL;

Symbol* SYM_STREAMS_STELLA_WITH_HTML_TAG = NULL;

Symbol* SYM_STREAMS_STELLA_WITH_HTML_TAG_EXPANDER = NULL;

Symbol* SYM_STREAMS_STELLA_WITH_XML_TAG = NULL;

Symbol* SYM_STREAMS_STELLA_WITH_XML_TAG_EXPANDER = NULL;

Keyword* KWD_STREAMS_CLOSED = NULL;

Keyword* KWD_STREAMS_OPEN = NULL;

Surrogate* SGT_STREAMS_STELLA_FILE_INPUT_STREAM = NULL;

Surrogate* SGT_STREAMS_STELLA_FILE_OUTPUT_STREAM = NULL;

Surrogate* SGT_STREAMS_STELLA_INPUT_STREAM = NULL;

Surrogate* SGT_STREAMS_STELLA_OUTPUT_STREAM = NULL;

Symbol* SYM_STREAMS_STELLA_THE_STREAM = NULL;

Surrogate* SGT_STREAMS_STELLA_S_EXPRESSION_ITERATOR = NULL;

Surrogate* SGT_STREAMS_STELLA_LINE_ITERATOR = NULL;

Surrogate* SGT_STREAMS_STELLA_CHARACTER_ITERATOR = NULL;

Symbol* SYM_STREAMS_STELLA_STARTUP_STREAMS = NULL;

Symbol* SYM_STREAMS_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
